import utils from '@/common/utils'

let heartbeatInterval = null; // 心跳定时器
let againTimer = null; //断线重连定时器
let times = 5; //重连和心跳间隔 秒
let socketObj = '';
let reconnecting = false;
let url = null;
let onReFn = null;
let onSucFn = null;
let onErrFn = null;
let onSendErrFn = null;
let onSendSucFn = null;
let activeStop = false;
/**
 * sockeUrl：websocet的地址
 * onReceive：消息监听的回调
 * onErrorEvent：抛出错误的回调，且弹窗连接失败的提示框
 * onErrorSucceed：抛出成功回调，主要用于隐藏连接失败的提示框
 * */
 
const sokcetInit = (sockeUrl, onReceive, onErrorEvent, onErrorSucceed) => {
	againTimer && clearInterval(againTimer)
	heartbeatInterval && clearInterval(heartbeatInterval)
	activeStop = false;
	reconnecting = false;
	
    url = sockeUrl;
    onReFn = onReceive;
    onErrFn = onErrorEvent;
    onSucFn = onErrorSucceed;
	
	let tokens = url+utils.getLoginStat();
	console.log('url:',tokens)
	socketObj && socketObj.close();
	socketObj = null;
	
	// 连接socket
	socketObj = uni.connectSocket({
	    url: tokens,
	    success(data) {
	        console.log("websocket开始连接ing");
	    },
	    fail: (err) => {
			console.log("websocket开始连接失败");
	    }
	});
	// 连接打开
	uni.onSocketOpen(function (res) {
		console.log('---WebSocket连接已打开！');
		onSucFn(res);
		// 监听收到信息
		uni.onSocketMessage(function (res) {
			console.log('---收到服务器内容');
			onReceive(res.data)
		});
		heartbeat();
	})
	// 监听连接失败
	uni.onSocketError(function (err) {
		console.log('---WebSocket连接打开失败，请检查！',err);
		onErrFn(err);
		if(!reconnecting&&!activeStop){
			reconnect();
		}
	})
	// // 监听连接关闭 -
	uni.onSocketClose(function (res) {
		console.log('---WebSocket 已关闭！');
		console.log(res)
		if(!reconnecting&&!activeStop){
			reconnect();
		}
	});
}

//向后端发送命令
const sendMsg = (msg, onSendErr, onSendSuc) => { 
	let msgs = JSON.stringify(msg);
	onSendErrFn = onSendErr;
	onSendSucFn = onSendSuc;
	//通过 WebSocket 连接发送数据
	uni.sendSocketMessage({
		data: msgs,
		success: function(){
			onSendSucFn && onSendSucFn(msg)
		},
		fail: function(){
			//reconnect()
			onSendErrFn && onSendErrFn(msg)
		}
	})
}
// 心跳
const heartbeat = () => {
	reconnecting = false;
	againTimer && clearInterval(againTimer)
	heartbeatInterval && clearInterval(heartbeatInterval)
	heartbeatInterval = setInterval(function(){
		// console.log('---心跳5s', new Date())
		sendMsg('ping')
	}, 1000 * times)
}

// 断线重连
const reconnect = () => {
	reconnecting = true;
	againTimer && clearInterval(againTimer)
	heartbeatInterval && clearInterval(heartbeatInterval)
	againTimer = setInterval(function(){
		console.log('---断线重连5s', new Date())
		sokcetInit(url, onReFn, onErrFn, onSucFn, onSendErrFn, onSendSucFn)
	}, 1000 * times)
}

// 关闭websocket【必须在实例销毁之前关闭,否则会是underfined错误】beforeDestroy() {websocetObj.stop();}
const stop = (res) => {
	againTimer && clearInterval(againTimer);
	heartbeatInterval && clearInterval(heartbeatInterval);
	activeStop = true;
	socketObj && socketObj.close();
	socketObj = null;
}

// const $debounce = function(fn, wait) {
//     let timer = null;
//     return function() {
//         if (timer !== null) {
//             clearTimeout(timer);
//         }
//         timer = setTimeout(fn, wait);
//     }
// }

export const websocetObj = {
    sokcetInit,
    stop,
    sendMsg
};
